package com.jkcredit.data.datastorage.service.resultInfoService;

import cn.afterturn.easypoi.excel.ExcelImportUtil;
import cn.afterturn.easypoi.excel.entity.ImportParams;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.google.common.util.concurrent.ThreadFactoryBuilder;
import com.jkcredit.data.datastorage.mapper.ResultInfoMapper;
import com.jkcredit.data.datastorage.model.dto.ResultFromFileDto;
import com.jkcredit.data.datastorage.model.dto.ResultInfoDto;
import com.jkcredit.data.datastorage.model.po.ResultInfoPo;
import com.jkcredit.data.datastorage.model.vo.CommonResponseVo;
import com.jkcredit.data.datastorage.service.identityInfoService.IdentityInfoService;
import com.jkcredit.data.datastorage.util.CommonUtil;
import com.jkcredit.data.datastorage.util.DESedeUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.transaction.TransactionDefinition;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.support.DefaultTransactionDefinition;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.InputStream;
import java.util.Date;
import java.util.List;
import java.util.concurrent.*;

/**
 * @description:
 * @author: xusonglin
 * @create: 2020/7/15 14:20
 * @version: V1.0
 **/
@Slf4j
@Service
public class ResultInfoServiceImpl implements ResultInfoService {
    @Value("${project.decryptKey}")
    private String decryptKey;
    @Resource
    ResultInfoMapper mapper;
    @Autowired
    IdentityInfoService identityInfoService;
    @Autowired
    private PlatformTransactionManager transactionManager;

    @Override
    public CommonResponseVo getResultInfo(String encodeParam) {
        String requestId = CommonUtil.getRequestId();

        JSONObject requestObject = JSON.parseObject(encodeParam);
        if (StringUtils.isBlank(requestObject.getString("param"))) {
            log.error("getIdentityInfo -- tempRequestId:{}, result:{}", requestId, "参数为空");
            return CommonResponseVo.failure("参数为空", requestId);
        }
        String requestParam = DESedeUtil.decryptMode(requestObject.getString("param"), decryptKey);

        // 参数校验
        if (StringUtils.isBlank(requestParam)) {
            log.error("getIdentityInfo -- tempRequestId:{}, result:{}", requestId, "参数解密失败");
            return CommonResponseVo.failure("参数解密失败", requestId);
        }

        ResultInfoDto resultInfoDto = JSON.toJavaObject(JSON.parseObject(requestParam), ResultInfoDto.class);
        log.info("getIdentityInfo -- tempRequestId:{},解密后参数:{}", requestId, resultInfoDto.toString());

        if (StringUtils.isBlank(resultInfoDto.getRequestId())) {
            log.error("getIdentityInfo -- tempRequestId:{}, result:{}", requestId, "请求参数requestId为空");
            return CommonResponseVo.failure("requestId为空", requestId);
        }

        requestId = resultInfoDto.getRequestId();
        ResultInfoPo resultInfoPo = mapper.getResultInfoByRequestId(requestId);
        if (resultInfoPo == null) {
            return CommonResponseVo.noResult("暂无结果", requestId);
        } else {
            return CommonResponseVo.success(resultInfoPo.getResult(), requestId);
        }
    }

    @Override
    public CommonResponseVo saveResultInfo(MultipartFile resultFile) {
        String requestId = CommonUtil.getRequestId();
        ImportParams params = new ImportParams();
        List<ResultFromFileDto> importList;
        // 读取excel
        try {
            InputStream inputStream = resultFile.getInputStream();
            importList = ExcelImportUtil.importExcel(inputStream, ResultFromFileDto.class, params);
        } catch (Exception e) {
            return CommonResponseVo.failure("读取excel失败", requestId);
        }

        int threadSize = 10;
        int count = importList.size() / 10;
        ThreadFactory namedThreadFactory = new ThreadFactoryBuilder().setNameFormat("thread-resultInfo-%d").build();

        ExecutorService executor = new ThreadPoolExecutor(threadSize, threadSize, 200L, TimeUnit.MILLISECONDS, new LinkedBlockingQueue<>(), namedThreadFactory);

        CountDownLatch countDownLatch = new CountDownLatch(threadSize);

        List<ResultFromFileDto> newList;
        try {
            for (int i = 0; i < threadSize; i++) {
                DefaultTransactionDefinition def = new DefaultTransactionDefinition();
                def.setPropagationBehavior(TransactionDefinition.PROPAGATION_REQUIRED);
                TransactionStatus status = transactionManager.getTransaction(def);
                if ((i + 1) == threadSize) {
                    int startIndex = (i * count);
                    int endIndex = importList.size();
                    newList = importList.subList(startIndex, endIndex);
                } else {
                    int startIndex = i * count;
                    int endIndex = (i + 1) * count;
                    newList = importList.subList(startIndex, endIndex);
                }

                List<ResultFromFileDto> finalNewList = newList;
                executor.execute(new Thread(() -> {
                    if (null != finalNewList) {
                        for (ResultFromFileDto dto : finalNewList) {
                            List<String> requestIds = identityInfoService.getRequestIdByIdInfo(dto.getIdCode(), dto.getName());
                            ResultInfoPo po = new ResultInfoPo();
                            po.setCreateTime(new Date());
                            po.setRequestId(requestIds.get(0));
                            po.setResult(dto.getResult());
                            mapper.insert(po);
                        }
                    }
                    countDownLatch.countDown();
                }));
                transactionManager.commit(status);
            }
        } catch (Exception e) {
            log.error("requestId:{}, result:{}, 异常：{}", requestId, "新增结果失败", e.getMessage());
            return CommonResponseVo.failure("新增结果失败", requestId);
        }

        return CommonResponseVo.success("操作完成", requestId);
    }
}
